/*
 *  tex2math.js
 *  
 *  Part of the jsMath package for mathematics on the web.
 *
 *  This file is a plugin that searches text wthin a web page
 *  for \(...\), \[...\], $...$ and $$...$$ and converts them to
 *  the appropriate <SPAN CLASS="math">...</SPAN> or
 *  <DIV CLASS="math">...</DIV> tags.
 *
 *  ---------------------------------------------------------------------
 *
 *  jsMath is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jsMath is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jsMath; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



jsMath.Add(jsMath,{
  
  ConvertTeX: function (element) {this.ConvertMath("TeX2math",element)},
  ConvertLaTeX: function (element) {this.ConvertMath("LaTeX2math",element)},
  
  ConvertMath: function (method,element,recurse) {
    if (!element) {
      if (recurse) return;
      element = document.body;
    }
    if (typeof(element) == 'string')
      {element = document.getElementById(element)}

    while (element) {
      if (element.nodeName == '#text') {
        if (!element.tagName || !element.tagName.match(/^(SCRIPT|STYLE|TEXTAREA)$/i)) {
          var call = this[method];
          element = call(element);
        }
      } else {
        this.ConvertMath(method,element.firstChild,1);
      }
      element = element.nextSibling;
    }
  },
  
  TeX2math: function (element) {
    var pattern = /((^|[^\\])(\\[^\[\(])*)(\\\((([^\\]|\\[^\)])*)\\\)|\\\[(([^\\]|\\[^\]])*)\\\]|(\$\$?)(([^$\\]|\\.)*)\9)/;
    var text; var result; var rest;
    while (result = pattern.exec(element.nodeValue.replace(/\n/g,' '))) {
      if (text = (result[5] || result[7] || result[10])) {
        var type = result[4].substr(0,2);
	var tag = jsMath.createMathTag(type,text);
        var math = element.splitText(result.index+result[1].length);
        rest = math.splitText(result[4].length);
        math.parentNode.replaceChild(tag,math);
      } else {
        rest = element.splitText(result.index+result[0].length);
        if (result[4] != '$$') {element.nodeValue = element.nodeValue.substr(0,result.index)}
      }
      element.nodeValue = element.nodeValue.replace(/\\\$/g,'$');
      element = rest;
    }
    element.nodeValue = element.nodeValue.replace(/\\\$/g,'$');
    return element;
  },
  
  LaTeX2math: function (element) {
    var pattern = /((^|[^\\])(\\[^\[\(])*)(\\\((([^\\]|\\.)*)\\\)|\\\[(([^\\]|\\.)*)\\\])/;
    var text; var result;
    while (result = pattern.exec(element.nodeValue.replace(/\n/g,' '))) {
      if (text = (result[5] || result[7])) {
        var type = result[4].substr(0,2);
        var tag = jsMath.createMathTag(type,text);
        var math = element.splitText(result.index+result[1].length);
        var rest = math.splitText(result[4].length);
        math.parentNode.replaceChild(tag,math);
      } else {
        rest = element.splitText(result.index+result[0].length);
        element.nodeValue = element.nodeValue.substr(0,result.index);
      }
      element.nodeValue = element.nodeValue.replace(/\\\$/g,'$');
      element = rest;
    }
    element.nodeValue = element.nodeValue.replace(/\\\$/g,'$');
    return element;
  },

  createMathTag: function (type,text) {
    type = (type == '\\[' || type == '$$')? "div" : "span";
    var tag = document.createElement(type);
    tag.className = "math";
    tag.innerHTML = text;
    return tag;
  },

  //
  //  MSIE won't let you insert a DIV within tags that are supposed to
  //  contain in-line data (like <P> or <SPAN>), so we have to fake it
  //  using SPAN tags that force the formatting to work like DIV.  We
  //  use a separate SPAN that is the full width of the containing
  //  item, and that has the margins from the div.typeset style
  //  and we name is jsMath.recenter to get jsMath to recenter it when
  //  it is typeset (HACK!!!)
  //
  MSIEcreateMathTag: function (type,text) {
    var tag = document.createElement("span");
    tag.className = "math";
    if (type == '\\[' || type == '$$') {
      tag.className = (jsMath.tex2math.center)? "jsMath.recenter": "";
      tag.style.width = "100%"; tag.style.margin = jsMath.tex2math.margin;
      text = '<SPAN CLASS="math">\\displaystyle{'+text+'}</SPAN>';
    }
    tag.innerHTML = text;
    return tag;
  }

});

if (jsMath.browser == 'MSIE') {
  jsMath.createMathTag = jsMath.MSIEcreateMathTag;
  jsMath.tex2math = {margin: "", center: 0};
  for (var i = 0; i < document.styleSheets.length; i++) {
    var rules = document.styleSheets[i].cssRules;
    if (!rules) {rules = document.styleSheets[i].rules}
    for (var j = 0; j < rules.length; j++) {
      if (rules[j].selectorText == 'DIV.typeset') {
        if (rules[j].style.margin != "") 
	  {jsMath.tex2math.margin = rules[j].style.margin}
	jsMath.tex2math.center =
          (rules[j].style.textAlign == 'center')? 1: 0;
      }
    }
  }
}
